# -*- coding: ISO-8859-15 -*-
"""Initiate and upload an image to an MediaWiki server

This program is part of "Wiki'it", a free software for
rapidly getting and modifiying screenshots from clipboards
and send them to a MediaWiki server

Copyright (c) 2008-2009 Jean-Philippe Ayanids <jp.ayanides@free.fr>
license : GPL v.3.0
see LICENSE for details about license
"""
import Tkinter
from PIL import Image,ImageTk

class ButtonBulle(Tkinter.Button):
	"""Classe de widget drive de la classe Button, et permettant d'afficher une infobulle"""
	def __init__(self,master=None,bgcolor="yellow",delay=750,justify="left",bulle="",icon=None,cnf={},**kw):
		"""bulle : texte de l'infobulle  afficher,
		delay : delai avant affichage de l'infobulle,
		icon : icone  afficher,
		bgcolor : couleur de fond de l'affichage
		justify : justification du texte de l'infobulle"""
		try:
			self.icon = ImageTk.PhotoImage(Image.open(icon))
			kw['image'] = self.icon
		except:
			self.icon = None
		Tkinter.Button.__init__(self, master=master, cnf=cnf, **kw)
		self.infobulle = InfoBulle(self,justify=justify,bgcolor=bgcolor,bulle=bulle,delay=delay)

class InfoBulle(Tkinter.Toplevel):
	def __init__(self,master=None,justify="left",bgcolor="yellow",bulle='',delay=750):
		Tkinter.Toplevel.__init__(self, master, bd=1, bg='black')
		self.delay = delay
		self.parent = master
		self.withdraw()
		self.overrideredirect(1)  # permet que la fentre n'ait pas de bord
		self.transient()	 
		l = Tkinter.Label(self, text=bulle, bg=bgcolor, justify=justify)
		l.update_idletasks()
		l.pack()
		l.update_idletasks()
		self.tipwidth = l.winfo_width()
		self.tipheight = l.winfo_height()
		self.parent.bind('<Enter>', self.delai)
		self.parent.bind('<Button-1>', self.efface)
		self.parent.bind('<Leave>', self.efface)
		
	def delai(self,event):
		"""On attend self.delay avant d'afficher l'infobulle"""
		self.action = self.parent.after(self.delay, self.affiche)

	def affiche(self):
		"""Affichage de l'infobulle"""	  
		self.update_idletasks()
		posX = self.parent.winfo_rootx()+self.parent.winfo_width()
		posY = self.parent.winfo_rooty()+self.parent.winfo_height()
		if posX + self.tipwidth > self.winfo_screenwidth():
			posX = posX-self.winfo_width()-self.tipwidth
		if posY + self.tipheight > self.winfo_screenheight():
			posY = posY-self.winfo_height()-self.tipheight
		self.geometry('+%d+%d'%(posX,posY))
		self.deiconify()

	## On cache l'infobulle ou on annule son affichage   
	def efface(self,event):
		self.withdraw()
		self.parent.after_cancel(self.action)


if __name__ == '__main__':
	root = Tkinter.Tk()
	root.title("Exemple de cration d'une classe InfoBulle")
	b1 = ButtonBulle(root,text="bouton 1",bulle="infobulle 1",delay=250,bgcolor="red")
	b2 = ButtonBulle(root,text="bouton 2",bulle="infobulle 2",delay=250,icon="images/minus.png")
	b1.pack()
	b2.pack()
	root.mainloop()
