#!/usr/bin/python
# coding: iso8859-1
"""configure the application Wiki'it

This program is part of "Wiki'it", a free software for
rapidly getting and modifiying screenshots from clipboards
and send them to a MediaWiki server

Copyright (c) 2008-2009 Jean-Philippe Ayanids <jp.ayanides@free.fr>
license : GPL v.3.0
see LICENSE for details about license
"""

import os
import ConfigParser
from tkSimpleDialog import *
from tkMessageBox import showerror


class ConfError(Exception): 
	pass

##############################################################
class Configuration:
	def __init__(self):
		"""Initialisation de tous les paramtres de configuration et de leur valeur par dfaut"""
		self.reset_all()

	def reset_all(self):
		"""remet tous les attributs  leur valeur par dfaut"""
		for k,v in self.default.items():
			setattr(self,k,v)

	def getdefault(self,arg):
		"""retourne la valeur par dfaut du paramtre"""
		if self.default.has_key(arg):
			return self.default[arg]
		else:
			raise ConfError

	def reset(self,*args):
		"""remet les attributs donns en paramtre  leur valeur par dfaut"""
		if not args:
			self.reset_all()
		else:
			for attr in args:
				if self.default.has_key(attr):
					setattr(self,attr,self.default[attr])

	def update(self,conf, **kw):
		"""update les valeurs de configuration depuis un 
		dictionnaire explicite conf ou implicite **kw"""
		if type(conf) == dict:
			for k,v in conf.items():
				self.set(k,v)
		else:
			raise TypeError
		if kw != None:
			for k,v in kw.items():
				self.set(k,v)

	def set(self,attr,val):
		"""attribue la valeur val  l'attribut attr, et lve une exception s'il n'existe pas"""
		if self.default.has_key(attr):
			setattr(self,attr,val)
		else:
			raise ConfError

	def get(self,attr=None):
		"""retourne un dictionnaire contenant toute la configuration, ou bien
		la valeur d'un attribut s'il est donn en paramtre et prsent dans la configuration.
		Lve une exception si l'attribut donn en paramtre n'existe pas"""
		if not attr:
			return dict((k,getattr(self,k)) for k in self.default.keys())
		else:
			if self.default.has_key(attr):
				return getattr(self,attr)
			else:
				raise ConfError

	def getsafe(self,attr):
		"""retourne la valeur de l'attribut donn en paramtre ou "" s'il n'existe pas"""
		if type(attr) != str:
			raise TypeError
		if self.default.has_key(attr):
			#print "%s -> %s" % (attr, getattr(self,attr))
			return getattr(self,attr)
		else:
			return ""

	def _print(self):
		"""affiche la configuration"""
		for attr in self.default.keys():
			print "%s : %s" % (attr,getattr(self,attr))

##############################################################
class ImageConf(Configuration):
	"""Spcialisation de la classe Configuration faite pour configurer l'image"""
	def __init__(self):
		self.default = {
			'filename':"", #nom configur de l'image
			'currFilename':"", #nom de la prochaine image
			'description':"", #descriptif de l'image ou de l'ensemble d'images
			'mode':"0", #mode nom direct (0) ou pattern (1)
			#pour le mode pattern:
			'pattern':None, #pattern  incrmenter pour former la valeur de filedest
			'num':1 #valeur de l'incrment  ajouter  pattern pour obtenir filedest
		}
		Configuration.__init__(self)

	def askConf(self,windowParent):
		"""Appelle la boite de dialogue de configuration et ragit en fonction du rsultat"""
		diag = ImageConfDialog(parent=windowParent,title="Configuration Wiki'it",config=self)
		if diag.result == None:
			print "abandon"
			return False
		else:
			filename = diag.result['filename']
			self.update(diag.result)
			#la politique de Mediawiki est de faire une capitalisation de la 1ere
			#lettre du nom. L'instruction suivante vite donc un warning.
			filename = filename[0].upper() + filename[1:]
			self.set("currFilename",filename + ".png")
			if self.getsafe("mode") == "1": #traitement du nom de fichier comme un pattern
				self.set("pattern",filename)
			else:
				self.set("pattern",None)
		del diag

	def incr(self): 
		"""incrmente le paramtre num"""
		self.num += 1

	def decr(self): 
		"""decrmente le paramtre num"""
		self.num -= 1

	def get(self,attr=None):
		# en mode incrmental, on met  jour currFilename
		if (attr == "currFilename" and self.mode == "1"): 
			self.currFilename = "%s-%i.png" % (self.pattern,self.num)
		return Configuration.get(self,attr)

##############################################################
class WikiitConf(Configuration):
	"""Spcialisation de la classe Configuration faite pour configurer le serveur"""
	def __init__(self):
		"""en cas d'ajout de paramtres il faudra aussi mettre  jour self.write 
		et la classe WikiitConfDialog qui fournit l'interface  la configuration"""
		configfile = ".wikiit.ini"
		self.file = "%s/%s" % (os.path.expanduser("~"),configfile)
		self.default={
			'host':"", 
			'port':"80", 
			'login':"", 
			'passwd':"", 
			'toolbar':"1",
			'back':"name" #choix entre "name" et "link" pour le retour de texte dans le presse papier
			}
		Configuration.__init__(self)

	def askConf(self,windowParent):
		"""Appelle la boite de dialogue de configuration et ragit en fonction du rsultat"""
		diag = WikiitConfDialog(parent=windowParent,title="Configuration Wiki'it",config=self)
		if diag.result == None:
			print "abandon"
			return False
		else:
			self.update(diag.result)
			self.write()
		del diag

	def read(self):
		"""Lecture d'un fichier de configuration au format INI et chargement des valeurs"""
		if (not os.path.exists(self.file)):
			self.reset_all()
			return False
		ini = ConfigParser.SafeConfigParser()
		ini.read(self.file)
		try:
			self.toolbar = ini.get("interface","toolbar")
		except (ConfigParser.NoOptionError, ConfigParser.NoSectionError):
			self.reset("toolbar")

		if not ini.has_section("mediawiki"):
			self.reset("host","port","login")
			return None

		for arg in ["host","login","port","back"]:
			try:
				setattr(self,arg,ini.get("mediawiki",arg))
			except ConfigParser.NoOptionError:
				self.reset(arg)

		self.reset("passwd")
		return True

	def write(self):
		"""Ecriture de la configuration courante dans un fichier de configuration au format INI"""
		ini = ConfigParser.SafeConfigParser()
		ini.add_section("mediawiki")
		ini.add_section("interface")
		ini.set("mediawiki","host",self.host)
		ini.set("mediawiki","login",self.login)
		ini.set("mediawiki","port",self.port)
		ini.set("mediawiki","back",self.back)
		ini.set("interface","toolbar",self.toolbar)
		try: 
			hfile = open(self.file,"w")
		except IOError:
			showerror("Configuration","Impossible d'enregistrer la configuration ; chemin incorrect:\n%s" % file)
		else:
			ini.write(hfile)
			hfile.close()

##############################################################
class WikiitConfDialog(Dialog):
	"""Classe de type Toplevel permettant de demander/proposer des 
	paramtres de configuration. Prend en paramtre un objet de classe 
	WikiitConf, et renvoie un dictionnaire de paramtres
	de configuration  jour."""
	def __init__(self, parent, title=None, config=None):
		"""initialise le dialogue, avec un objet de configuration (class=WikiitConf)"""
		if config.__class__ != WikiitConf:
			raise TypeError
		self.preconf = config
		self.fields = {}
		self.fields['toolbar'] = StringVar()
		self.fields['back'] = StringVar()
		Dialog.__init__(self,parent,title=title)
	
	def body(self,master):
	#labels et placement des labels
		Label(master, text="barre d'icones").grid(row=0,sticky=W)
		Label(master, text="serveur Mediawiki").grid(row=1,sticky=W)
		Label(master, text="port").grid(row=2,sticky=W)
		Label(master, text="identifiant de connexion").grid(row=3,sticky=W)
		Label(master, text="lien  coller").grid(row=4,sticky=W)
	#champs
		self.fields['host']   = Entry(master)
		self.fields['port']   = Entry(master)
		self.fields['login']  = Entry(master)
		self.back1 = Radiobutton(master, text="mode visuel (nom de fichier)", variable=self.fields['back'], value="name")
		self.back2 = Radiobutton(master, text="mode texte (lien wiki)", variable=self.fields['back'], value="link")
		self.toolbar = Checkbutton(master,variable=self.fields['toolbar'],onvalue="1",offvalue="0")
	#placement des champs
		self.toolbar.grid(sticky=W,row=0,column=1)
		self.fields['host'].grid(row=1, column=1)
		self.fields['port'].grid(row=2, column=1)
		self.fields['login'].grid(row=3,column=1)
		self.back1.grid(sticky=W,row=4,column=1)
		self.back2.grid(sticky=W,row=5,column=1)
	#prinsertion des valeurs dans les champs
		self.fields['host'].insert(END,self.preconf.getsafe('host'))
		self.fields['port'].insert(END,self.preconf.getsafe('port'))
		self.fields['login'].insert(END,self.preconf.getsafe('login'))
		if self.preconf.getsafe('toolbar') == "1": self.toolbar.select()
		else: self.toolbar.deselect()
		if self.preconf.getsafe('back') == "name": self.back1.select()
		else: self.back1.deselect()
		if self.preconf.getsafe('back') == "link": self.back2.select()
		else: self.back2.deselect()

	#focus initial
		return self.fields['host']

	def validate(self):
		return True

	def apply(self):
		self.result = dict((k,self.fields[k].get()) for k in self.fields.keys())

##############################################################
class ImageConfDialog(Dialog):
	"""classe de type TopLevel permettant d'afficher des zones pour 
	la saisie d'un nom et d'un descriptif, et des options d'utilisation."""
	def __init__(self, parent, title=None, config=None):
		"""initialise le dialogue, avec un objet de configuration (class=ImageConf)"""
		if config.__class__ != ImageConf:
			raise TypeError
		self.preconf = config
		self.fields = {}
		self.fields['mode'] = StringVar()
		Dialog.__init__(self,parent,title=title)

	def initShow(self):
		if self.fields["mode"].get() == "1":
			self.iniLabel.grid(row=1, column=2)
			self.fields['num'].grid(sticky=E, row=1, column=3)
		else:
			self.iniLabel.grid_forget()
			self.fields['num'].grid_forget()
			
	def body(self, master):
	#labels et placement des labels
		Label(master, text="nom de l'image (sans extension)").grid(row=0)
		Label(master, text="le nom est un pattern incrmentable").grid(row=1)
		Label(master, text="descriptif (facultatif)").grid(row=2)
		self.iniLabel = Label(master, text="1er index")
	#champs (le focus de la zone de texte est dplac vers l'arrire)
		self.fields['description'] = Text(master,height=3, width=21)
		self.mode = Checkbutton(master,takefocus=1,variable=self.fields['mode'],
			onvalue="1",offvalue="0",command=self.initShow)
		self.fields['num'] = Spinbox(master, from_=1, to=99, width=2)
		self.fields['filename'] = Entry(master,width=21)
	#placement des champs
		self.fields['filename'].grid(row=0, column=1, columnspan=3)
		self.mode.grid(sticky=W,row=1, column=1)
		self.fields['description'].grid(row=2, column=1, columnspan=3)
	#prinsertion des valeurs dans les champs
		self.fields['filename'].insert(END,self.preconf.getsafe('filename'))
		self.fields['num'].delete(0,END)
		self.fields['num'].insert(END,self.preconf.getsafe('num'))
		self.fields['description'].insert(END,self.preconf.getsafe('description'))
		if self.preconf.get('mode') == "1": 
			self.mode.select()
		self.initShow()
	#le premier champ est mis en video inverse
		self.fields['filename'].selection_range(0, END) 
	#focus initial
		return self.fields['filename']

	def apply(self):
		self.result = dict((k,self.fields[k].get()) for k in ["filename","mode","num"])
		desc = self.fields['description'].get(1.0,END)
		if desc == '\n': 
			desc = ""
		self.result["description"] = desc

	def validate(self):
		filename = self.fields["filename"].get()
		if not filename:
			showerror("Nom de l'image","Il faut fournir un nom de fichier!")
			return False
		elif filename[0] == ".":
			showerror("Nom de l'image","Il faut fournir un nom de fichier valide !")
			return False
		if filename.endswith('.png') or filename.endswith('.gif'):
			showerror("Nom de l'image","Le nom de l'image ne doit pas contenir l'extension !")
			return False
		if self.fields["mode"].get() == "1":
			try:
				int(self.fields["num"].get())
			except ValueError:
				showerror("Premier index","L'index doit tre un entier")
				return False
		else:
			self.fields['num'].delete(0,END)
			self.fields['num'].insert(END,self.preconf.getdefault('num'))
		return True

#########################################################
if __name__ == "__main__":
	conf = WikiitConf()
	conf.passwd = "toto"
	conf.port = "81"
	conf._print()
	conf.reset("passwd","port")
	print ""
	conf._print()
	print ""
	print conf.get()
	print conf.get("port")
