# coding: iso8859-1
"""Initiate and upload an image to an MediaWiki server

This program is part of "Wiki'it", a free software for
rapidly getting and modifiying screenshots from clipboards
and send them to a MediaWiki server

Copyright (c) 2008-2009 Jean-Philippe Ayanids <jp.ayanides@free.fr>
license : GPL v.3.0
see LICENSE for details about license
"""

import sys
import urllib3, Cookie

class MWError(Exception): pass
class MWhttpError(MWError): pass
class MWsessionError(MWError): pass
class MWpageError(MWError): pass
class MWauthError(MWError): pass
class MWuploadError(MWError): pass
class MWuploadExist(MWError): pass

class mediawiki:
	"""open a connection with a mediawiki server, identify user and upload
	an image"""
	#ouvre une connection sur un serveur wikimedia, s'identifie et upload une image
	def __init__(self,host,port=80,timeout=8):
		"""creating connection"""
		self.host,self.port = (host,port)
		self.connection = urllib3.HTTPConnectionPool(self.host,self.port,timeout=timeout)
		self.status = 0 #status of connection and processes

	def createCookie(self,simple_cookie):
		"""function which can generate a cookie to send from a received cookie (when
		not founded into Cookie class (Cookie.SimpleCookie)"""
		#fonction permettant de crer un cookie  envoyer  partir d'un cookie reu 
		#(car non trouv dans la classe Cookie (Cookie.SimpleCookie))
		cookie=[]
		for k,v in simple_cookie.items():
			cookie.append("%s=%s" % (k,simple_cookie.get(k).value))
		return '; '.join(cookie)

	def login(self,login,password):
		"""Initiating HTTP connection in order to get a session cookie and then
		an identity cookie"""
		#Initiation de la connection HTTP pour rcuprer un cookie de session 
		#puis un jeton d'identit
		page1 = "/index.php/Special:Userlogin"
		page2 = "/index.php?title=Special:Userlogin&action=submitlogin&type=login"
		headers1 = {
			"User-Agent": "Wikiit",
			#"User-Agent": "Mozilla/5.0",
			"Content-type": "text/html",
			"Connection":"Keep-Alive",
			"Host":self.host,
			"Pragma":"no-cache",
			"Cache-control":"no-cache",
			"Accept":"text/html, image/jpeg, image/png, text/*, image/*, */*",
			"charset":"utf-8" # added mercredi 17 mars 2010, 16:01:46 (UTC+0100)
		}

		try:
			ans1 = self.connection.get_url(page1,{},headers1)
		except (urllib3.MaxRetryError,urllib3.TimeoutError), reason:
			#raise MWhttpError, "Erreur de connection pendant l'ouverture de session:\n%s" % reason
			raise MWhttpError, "Connection error during session opening:\n%s" % reason
			return

		cookie1 = Cookie.SimpleCookie()
		try:
			cookie1.load(ans1.getheader("set-cookie")) 
		except TypeError: #I'm not sure to know how to get cookies and what says RFC
			try:
				cookie1.load(ans1.getheader("Set-cookie")) 
			except:
				#print "step 1, no cookie"
				pass

		if (ans1.status == 200 and cookie1.has_key('wikidb_session')):
			print "Opening session successful."
		else:
			raise MWsessionError, "Error when opening session."
			return

		#if all is ok, let's go on
		fields2 = {
			"wpName":unicode(login).encode('utf-8'),
			"wpPassword":unicode(password).encode('utf-8'),
			"wpRemember":"1",
			"wpLoginattempt":"Identification"
		}

		headers2 = {
			"User-Agent": "Wikiit",
			#"User-Agent": "Mozilla/5.0",
			"Content-type": "application/x-www-form-urlencoded",
			"Referer":"http://%s/%s" % (self.host,page1),
			"Connection":"Keep-Alive",
			"Host":self.host,
			"Pragma":"no-cache",
			"Cache-control":"no-cache",
			"Accept":"text/html, image/jpeg, image/png, text/*, image/*, */*",
			"Accept-Language": "en",
			"Cookie": self.createCookie(cookie1),
			"charset":"utf-8" # added wednesday 17 march 2010, 16:01:46 (UTC+0100)
		}
		try:
			ans2 = self.connection.post_url(page2,fields2,headers2)
		except (urllib3.MaxRetryError,urllib3.TimeoutError), reason:
			raise MWhttpError, "Erreur de connection pendant l'identification:\n%s" % reason
			return

		#getting token wikidbToken (and wikidbUserID, wikidbUserName)
		cookie2 = Cookie.SimpleCookie()
		try:
			cookie2.load(ans2.getheader("set-cookie")) 
		except TypeError: #I'm not sure to know how to get cookies and what says RFC 2109
			try:
				cookie2.load(ans1.getheader("Set-cookie")) 
			except:
				#print "step 2, no cookie"
				pass

		if ans2.status != 200:
			raise MWpageError, "Error when running identification process."
			return
		elif not cookie2.has_key('wikidbToken'):
			raise MWauthError, "Incorrect login or password"
			return
		else:
			print "Authentification successful."

		self.cookie = Cookie.SimpleCookie(cookie1)
		self.cookie.update(cookie2)

	def upload(self,filefrom,filedest,description,follow=None,warning=None):
		"""Process which allow to upload an image. By default, warning=None allow to take into account all warnings"""
		#procdure permettant d'uploader une image. Par dfaut, warning=None permet de prendre en compte tous les avertissements
		page = "/index.php/Special:Upload"
		image = open(filefrom,"rb")
		content = image.read()
		image.close()
		fields = {
		    'wpUploadFile':(filefrom, content), #file to send
		    'wpSourceType':'file',
		    'wpDestFile':unicode(filedest).encode('utf-8'),
		    'wpUploadDescription':unicode(description).encode('utf-8'),
		    'wpUpload':"Copy a file"
		}
		if follow == "true": fields['wpWatchthis'] = 'true' # watch this page
		if warning == "true": fields['wpIgnoreWarning'] = 'true'  # ignore all warnings

		headers = {
			#"User-Agent": "Mozilla/5.0",
			"User-Agent": "Wikiit",
			"Content-type": "text/html",
			"Referer":"http://%s/%s" % (self.host,page),
			"Connection":"Keep-Alive",
			"Host":self.host,
			"Pragma":"no-cache",
			"Cache-control":"no-cache",
			"Accept":"text/html, image/jpeg, image/png, text/*, image/*, */*",
			"Cookie":self.createCookie(self.cookie),
			"Accept-Charset": "iso-8859-15, utf-8;q=0.5, *;q=0.5",
			"Accept-Language": "fr, en",
			"charset":"utf-8"
		}
		try:
			ans = self.connection.post_url(page,fields,headers)
		except (urllib3.MaxRetryError,urllib3.TimeoutError), reason:
			raise MWhttpError, "Connection error when opening session:\n%s" % reason
			return

		self.saveAns(ans.data,"server-response.html")
		if ans.status != 200:
			print "Error when uploading file!"
			raise MWuploadError, "Error when transfering image."
		else:
			if ans.data.find("<form id='uploadwarning'") != -1: # une image de ce nom existe dj !
				raise MWuploadExist, "There is already an image with that filename."
				return
				#TODO : IT WON'T WORK IF LANGUAGE IS NOT DEFINED AS FRENCH. MUST BE FIXED !
			if (ans.data.find('<title>Erreur</title>') != -1) or (ans.data.find('<h2>Attention !</h2>') != -1):
			#FIXME : it would be great to list name of images which already exist, like FCKeditor
				print "Error when recording file!"
				raise MWuploadError, "Server-side error (double, etc.) Proceed manually."
				return

	def checkConn(self):
		"""verify if session is still valid, by reading cookies. Return True if yes, and False if not."""
		#Vrifie via la lecture des cookies si la session est encore valide. Retourne True si oui, et False sinon
 		#FIXME :  coder en exploitant le cookie de session
		return True #connection still opened

	def saveAns(self,data,filename):
		"""For debug needs, we call this function to write answer in a file
		named filename, before continuing"""
		#pour les besoins du debug, on appelle cette fonction pour crire la rponse dans un fichier de nom filename avant de continue
		file = open(filename,"w")
		file.write(data)
		file.close()


if __name__ == '__main__':
	from tkMessageBox import showerror, showwarning
	host="wikipro-test.prosodie"
	login, password = ("test","test")
	filefrom="images/wikiit_256x256.png"
	filedest="mytestimage.png"
	description="test description text"

	mw = mediawikiUpload(host)
	try: #tentative d'identification
		mw.login(login,password)
	except MWautError, msg:
		showwarning("Connection",msg)
		sys.exit(1)
	except MWError, msg:
		showerror("Connection",msg)
		sys.exit(1)
	try: #tentative d'upload
		mw.upload(filefrom,filedest,description)
	except MWuploadExist, msg:
		showwarning("Upload",msg)
		sys.exit(1)
	except MWuploadError, msg:
		showerror("Upload",msg)
		sys.exit(1)
