#!/usr/bin/python
# coding: iso8859-1

# see http://pylonsbook.com/en/1.1/unicode.html for unicode issues with password
"""main file of Wiki'it

This program is part of "Wiki'it", a free software for
rapidly getting and modifiying screenshots from clipboards
and upload them to a MediaWiki server

Copyright (c) 2008-2009 Jean-Philippe Ayanids <jp.ayanides@free.fr>
license : GPL v.3.0
see COPYRIGHT for details about license"""

version = "0.9.4"

#modules gnraux
from sys import path, platform
import os
from tempfile import mkstemp
import ConfigParser
import logging

#toolkit graphique
from Tkinter import *
from tkFileDialog import asksaveasfilename,askopenfilename 
from tkMessageBox import *
from tkSimpleDialog import *
from tkColorChooser import askcolor

#modules locaux
path.append("./")
from buttonbulle import *
from mediawiki import *
from configuration import *

#manipulation des images
import PIL.Image as PILImage
from PIL import ImageTk, ImageDraw
from ImageColor import *

#modules win32
if sys.platform == "win32": 
	import win32clipboard 
	from cStringIO import StringIO
	from PIL import BmpImagePlugin


class WikiClip(Tk):

	def __init__(self):
		Tk.__init__(self) #creation de la zone fentre de haut niveau gr par le WM
		global version

		if sys.platform == "win32": 
			self.iconbitmap("icons/wikiit.ico")

		self.minsize(width=500,height=100) #la largeur de 500 permet de ne pas rogner la toolbar 

		#init de la configuration 
		self.conf = WikiitConf()
		self.conf.read()
		#init de la configuration des images
		self.confImage = ImageConf()
		#init de la connexion
		self.mw = None 
		self.delay = 250 #delai d'attente en ms avant l'affichage de l'infobulle
		self.title("Wiki'it v.%s" %version)

		#pour les encadrements colors
		self.widthGraph = 2 # paisseur des traits de dessin
		self.widthGraphText = "paisseur: %i" %self.widthGraph # label indiquant l'paisseur des traits
		self.colors =  ["red","blue","black","gray","green","cyan","skyblue","gold","yellow"]
		self.iterColor = iter(self.colors)  # itrateur des couleurs de dessin
		self.graphColor = self.iterColor.next() # couleur actuelle de dessin

		self.x1,self.y1,self.x2,self.y2 = (0,0,0,0)
		self.ruban = None # init du rectangle de selection

		self.old_imageBitmap = None #image de backup
		self.imageBitmap = None #image bitmap sur laquelle on va travailler
		self.imagePhoto = None  #imagePhoto utilise pour plaquer l'image sur le canevas
		self.imageItem = None   #item d'image place sur le canevas
		#FIXME : il faudrait supprimer ce problme de gap
		self.gap = {"x": 0, "y": 0} #gap entre la taille demande pour le canevas et sa taille obtenue
		self.toolbarHeight = 0 #hauteur de la boite d'outils, selon qu'elle est visible ou non

		#creation du menu
		self.createMenu(parent=self) #positionne le menu dans son frame
		self.config(menu=self.menu)

		#creation du canevas
		self.canvas = Canvas(self)
		self.xScrollbar = Scrollbar(self, width=16, orient=HORIZONTAL, command=self.canvas.xview)
		self.yScrollbar = Scrollbar(self, width=16, orient=VERTICAL, command=self.canvas.yview)
		#on connecte les ascenseurs au canevas
		self.canvas.configure(xscrollcommand=self.xScrollbar.set, yscrollcommand=self.yScrollbar.set)
                              #xscrollincrement=15, yscrollincrement=15)
		#on place
		self.canvas.grid(row=0,column=0)
		self.yScrollbar.grid(row=0,column=1,sticky=N+S)
		self.xScrollbar.grid(row=1,column=0,sticky=E+W)
		self.grid_rowconfigure(0, weight=1)
		self.grid_columnconfigure(0, weight=1)

		#creation de la barre d'outils
		self.toolbar = Frame(self, borderwidth=1)#, background="red") #hauteur=70 (linux)
		self.createToolbar(parent=self.toolbar) #positionne les icones dans leur frame

		self.openClipboard() #chargement de l'image et placement sur le canevas
		self.toolbarToggle(show=self.conf.toolbar) #placement ou non en fonction de la configuration
		#self.maxsize(width=self.topFrame.winfo_width()   + 8, height=self.topFrame.winfo_height() + 111)

		# ajout des binding souris et clavier
		if sys.platform == "win32":
			self.bind("<MouseWheel>", self.scrollVert)
			self.xScrollbar.bind("<MouseWheel>", self.scrollHori)
		else:
			self.canvas.bind("<Button-4>", self.scrollUp)
			self.canvas.bind("<Button-5>", self.scrollDown)
			self.yScrollbar.bind("<Button-4>", self.scrollUp)
			self.yScrollbar.bind("<Button-5>", self.scrollDown)
			self.xScrollbar.bind("<Button-4>", self.scrollLeft)
			self.xScrollbar.bind("<Button-5>", self.scrollRight)

		self.canvas.bind("<Button-1>", self.initCadre)
		self.canvas.bind("<Button1-Motion>", self.traceCrop)
		self.canvas.bind("<Button1-ButtonRelease>", self.cropCadre)

		self.canvas.bind("<Button-3>", self.initCadre)
		self.canvas.bind("<Button3-Motion>", self.traceCadreGraph)
		self.canvas.bind("<Button3-ButtonRelease>", self.drawGraph)

		self.bind("<Control-v>", self.openClipboard)
		self.bind("<Control-Return>", self.send)
		self.bind("<Control-s>", self.save)
		self.bind("<Control-q>", self.exit)
		self.bind("<Control-z>", self.undo)
		self.bind("+", self.incWidthGraph)
		self.bind("-", self.decWidthGraph)
		#self.bind("+", func = (lambda : self.setWidthGraph(sign="+")))
		#self.bind("-", func = (lambda : self.setWidthGraph(sign="-")))
		self.bind("-", self.decWidthGraph)
		self.bind("<Configure>",self.reconfigure)
		self.bind("<t>",self.toolbarToggle)
		self.bind("<c>",self.changeColor)
		self.unbind_all("<Print>")

		self.toolbar.bind("<Map>", self.toolbarMap)
		self.toolbar.bind("<Unmap>", self.toolbarUnmap)
		#pour le debug
		self.bind("<p>",self.getsizes)

		self.update_idletasks()
		#on vrifie l'existence d'une configuration :
		#TODO: qu'est ce que a fait l ?
		if (self.conf.get('host') == "") or (self.conf.get('port') == ""):
			res = self.askWikiitConf()
			if res == None:
				print "abandon"
				self.exit()
				return

	def toolbarMap(self,event):
		#FIXME :  coder en remplacement des callbacks quivalents
		pass
		#print "mapping toolbar",
		#print self.toolbar.winfo_height()
	def toolbarUnmap(self,event):
		#FIXME :  coder en remplacement des callbacks quivalents
		pass
		#print "unmapping toolbar",
		#print self.toolbar.winfo_height()

	def reconfigure(self,event=None,bitmap=None):
		"""Redimensionne les widgets de la fentre principale self en fonction de la taille
		de l'image, de la taille de la fentre principale, de la prsence ou non de la barre
		d'outils, et affiche ou cache les ascenseurs selon les conditions. Si l'argument
		bitmap est fourni, commence par afficher sur le canvas le nouveau bitmap en lieu et
		place de l'ancien qui est alors backup.

		Attention, il y a un gap entre la taille du canvas et la taille attribue au canvas
		par le windows manager. Il vaut 2 sous KDE et 4 sous Windows. Il faut en tenir compte."""

		if bitmap:
			if not PILImage.isImageType(bitmap):
				raise TypeError
			#le boutton sendButton repasse au rouge	
			self.sendButton.configure(background='red')
			self.backupImage()
			self.imageBitmap = bitmap
			self.imagePhoto = ImageTk.PhotoImage(self.imageBitmap)
			self.canvasHeight = self.imagePhoto.height() #+ 8
			self.canvasWidth  = self.imagePhoto.width()  #+ 8
			self.canvas.delete(self.imageItem) #suppression de l'ancienne image du canvas
			self.canvas.configure(width=self.canvasWidth, height=self.canvasHeight,
					scrollregion=(0, 0, self.canvasWidth, self.canvasHeight))
			self.imageItem = self.canvas.create_image(self.canvasWidth//2,self.canvasHeight//2, image=self.imagePhoto)
			self.update_idletasks()
			# Ca, c'est une super bonne source de bug !
			#self.gap = {"x": (self.canvas.winfo_width()  - self.canvasWidth),
			#i            "y": (self.canvas.winfo_height() - self.canvasHeight)}
			if sys.platform == "win32": self.gap = {"x": 4, "y": 4}
			else:                       self.gap = {"x": 2, "y": 2}
			self.canvasReconfig()

		if event:
			if event.widget == self:
				self.canvasReconfig(event)
				#print "tk         event:       h:   %i,\tL:   %i" % (event.height,event.width)
			#elif event.widget == self.toolbar :
				#print "toolbar    event",event.state
			#elif event.widget == self.canvas :     
			#	print "canvas     event:       h:   %i,\tL:   %i" % (event.height,event.width)
			#elif event.widget == self.xScrollbar : print "xScrollbar event"
			#elif event.widget == self.yScrollbar : print "yScrollbar event"
			#else:                                  print "unknown    event"

	def canvasReconfig(self,event=None):
		"""redimensionne le canvas en fonction de la taille de l'image, de la taille 
		de la fentre principale, et rgle la prsence des ascenseurs du canvas"""
		#FIXME : il faudrait amliorer le code. cf
		#http://www.pythonware.com/library/tkinter/introduction/basic-widget-methods.htm
		self.update_idletasks()
		if self.xScrollbar.get()[0] == 0.0 and self.xScrollbar.get()[1] == 1.0:
			self.xScrollbar.grid_forget()
			xScrollbarHeight = 0
		else:
			self.xScrollbar.grid(row=1,column=0,sticky=E+W)
			xScrollbarHeight = self.xScrollbar.winfo_height()
		if self.yScrollbar.get()[0] == 0.0 and self.yScrollbar.get()[1] == 1.0:
			self.yScrollbar.grid_forget()
			yScrollbarWidth = 0
		else:
			self.yScrollbar.grid(row=0,column=1,sticky=N+S)
			yScrollbarWidth = self.yScrollbar.winfo_width()
		# Dy est la diffrence entre la hauteur de la fentre principale 
		# et les hauteurs des diffrents widgets
		Dy = self.winfo_height() \
		   - self.canvas.winfo_height() \
		   - xScrollbarHeight \
		   - self.toolbarHeight 
		# Dx est la diffrence entre la largeur de la fentre principale 
		# et les largeurs des diffrents widgets
		Dx = self.winfo_width() \
		   - yScrollbarWidth \
		   - self.canvas.winfo_width()

		if Dy < 0 :
			#print "reducing canvas height (%i) by %i" %(self.canvas.winfo_height(),-Dy)
			self.canvas.configure(height=self.canvas.winfo_height() - self.gap["y"] + Dy)
			self.update_idletasks()
			#print "new height : %i" % self.canvas.winfo_height()

		if Dx < 0:
			#print "reducing canvas width (%i) by %i" %(self.canvas.winfo_width(),-Dx)
			self.canvas.configure(width=self.canvas.winfo_width() - self.gap["x"] + Dx)
			self.update_idletasks()
			#print "new width : %i" % self.canvas.winfo_width()

		if Dy > 0 and self.canvas.winfo_height() < self.canvasHeight + self.gap["y"]:
			#print "increasing canvas height (%i) by %i" %(self.canvas.winfo_height(),Dy)
			self.canvas.configure(height=min(self.canvas.winfo_height() - self.gap["y"] + Dy,self.canvasHeight))
			self.update_idletasks()
			#print "new height : %i" % self.canvas.winfo_height()

		if Dx > 0 and self.canvas.winfo_width() < self.canvasWidth + self.gap["x"]:
			#print "increasing canvas width (%i) by %i" %(self.canvas.winfo_width(),Dx)
			self.canvas.configure(width=min(self.canvas.winfo_width() - self.gap["x"] + Dx, self.canvasWidth))
			self.update_idletasks()
			#print "new width : %i" % self.canvas.winfo_width()

	def openClipboard(self, event=None):
		"""Tente de charger l'image depuis le clipboard"""
		print "loading image"
		#on est oblig de crer un objet Image et un objet ImageTk, car seul le 1er 
		#peut sauver l'objet alors que le canvas ne peut afficher que le 2e
		if sys.platform == "win32":
			win32clipboard.OpenClipboard()
			#print "format du contenu du clipboard %i" % win32clipboard.EnumClipboardFormats()
			#FIXME : quel est le format 49645 ?
			if win32clipboard.IsClipboardFormatAvailable(win32clipboard.CF_BITMAP):
				data = win32clipboard.GetClipboardData(win32clipboard.CF_DIB) #la ruse !!
				self.reconfigure(bitmap=BmpImagePlugin.DibImageFile(StringIO(data)))
			else: # clipboard vide ou ne contenant pas d'image
				print "Pas d'image bitmap dans le clipboard"
				self.openFile(filename="images/void.png")
			win32clipboard.CloseClipboard() 
		else: #on charge une simple image de test
			self.openFile(filename="images/wikiit_256x256.png")

	def initCadre(self, event):
		"""Initialise les coordonnes self.x1,self.y1 sur le canvas  celle de l'vnement"""
		# canvas x and y take the screen coords from the event and translate
		# them into the coordinate system of the canvas object
		self.x1,self.y1 = (self.canvas.canvasx(event.x),self.canvas.canvasy(event.y))
		#print "event x et y: %i, %i ; x1 et y1: %i, %i" %(event.x,event.y,self.x1,self.y1)

	def slide(self, event):
		"""fait glisser le canevas pour que la coordonne de l'vnement reste visible autant
		que possible"""
		if event.x + 3 > self.canvas.winfo_width():  self.canvas.xview_scroll(1, "units")
		if event.y + 3 > self.canvas.winfo_height(): self.canvas.yview_scroll(1, "units")
		if event.x - 3 < 0:                          self.canvas.xview_scroll(-1, "units")
		if event.y - 3 < 0:                          self.canvas.yview_scroll(-1, "units")

	def traceCrop(self, event):
		"""Trace un cadre de slection de l'image"""
		# canvas x and y take the screen coords from the event and translate
		# them into the coordinate system of the canvas object
		self.x2,self.y2 = (self.canvas.canvasx(event.x),self.canvas.canvasy(event.y))

		if (self.x1 != self.x2) and (self.y1 != self.y2): #si la zone est gomtriquement non vide
			self.slide(event)
			#on supprime le rectangle prcdent
			try:    self.canvas.delete(self.ruban)
			except: pass
			self.ruban = self.canvas.create_rectangle(self.x1, self.y1, self.x2, self.y2,outline="black",width=1)
			self.update_idletasks()

	def traceCadreGraph(self, event):
		"""Trace sur l'image des encadrements vectoriels colors de mise en valeur graphique"""
		self.x2 = self.canvas.canvasx(event.x)
		self.y2 = self.canvas.canvasy(event.y)
		if (self.x1 != self.x2)  and (self.y1 != self.y2): #si la zone est gomtriquement non vide
			self.slide(event)
			try:
				self.canvas.delete(self.cadre)#on supprime le rectangle prcdent
			except: pass
			self.cadre = self.canvas.create_rectangle(self.x1,self.y1,self.x2,self.y2,
				outline=self.graphColor,width=self.widthGraph)
			self.update_idletasks()

	def cadreUtil(self):
		"""Dtermine les coordonnes du cadre slectionn, en le restreignant  la zone de l'image.
		Si tout le cadre est hors des limites de la zone de l'image, il retourne None.
		Sinon, il retourne le quadruplet (x1,y1,x2,y2)"""
		if (self.x1 == self.x2) or (self.y1 == self.y2): # si le rectangle est gomtriquement vide
			return None
		bbox = self.canvas.bbox(self.imageItem) #boite au plus proche de l'image
		# on recalcule les coordonnes du rectangle pour le restreindre  la zone de l'image
		# s'il la dpasse, remettre dans l'ordre les x1<x2 et y1<y2 (sinon bug sur le paste)
		# et exprimer ces coordonnes par rapport  l'image et non au canvas 
		# (grace  la soustraction du dernier terme)
		if self.x1 < self.x2:
			x1 = int(max(self.x1,bbox[0]) - bbox[0])
			x2 = int(min(self.x2,bbox[2]) - bbox[0])
		else:
			x1 = int(max(self.x2,bbox[0]) - bbox[0]) 
			x2 = int(min(self.x1,bbox[2]) - bbox[0])
		if self.y1 < self.y2:
			y1 = int(max(self.y1,bbox[1]) - bbox[1])
			y2 = int(min(self.y2,bbox[3]) - bbox[1])
		else:
			y1 = int(max(self.y2,bbox[1]) - bbox[1])
			y2 = int(min(self.y1,bbox[3]) - bbox[1])
			
		box = (x1,y1,x2,y2) #zone de slection utile, en coordonnes image
		x,y = (x2 - x1, y2 - y1) #dimensions de la zone
		#si  prsent il se trouve que le cadre est hors zone ou vide:
		if (x <= 0 or y <= 0):
			return None
		else:
			#print (x1,y1,x2,y2)
			#print (self.x1,self.y1,self.x2,self.y2)
			return (x1,y1,x2,y2)

	def cropCadre(self, event):
		"""Appelle les coordonnes du cadre, copie la zone de l'image situe
		dans la boite, efface l'ancienne image, et enfin colle la nouvelle image, au centre du canevas."""
		self.x2,self.y2 = (self.canvas.canvasx(event.x),self.canvas.canvasy(event.y))
		box = self.cadreUtil() #zone de slection utile, en coordonnes image
		if (box == None):
			self.canvas.delete(self.ruban) # suppression du rectangle de slection
			return #et on ne fait rien
		x1,y1,x2,y2 = box
		x,y = (x2 - x1, y2 - y1) #dimensions de la zone
		#print x,y
		self.zone = self.imageBitmap.crop(box)
		#on backup l'ancienne image pour le retour arrire
		self.backupImage()
		bitmap = PILImage.new("RGB",(x,y))
		bitmap.paste(self.zone,(0,0,x,y))
		self.canvas.delete(self.ruban) # suppression du rectangle de slection
		self.reconfigure(bitmap=bitmap)

	def drawGraph(self,event):
		"""Backup l'ancienne image, efface le cadre de mise en valeur vectoriel et 
		le remplace par un cadre de mise en valeur dessin sur l'image"""
		box = self.cadreUtil() #zone de slection utile, en coordonnes image
		if (box == None):
			try: self.canvas.delete(self.cadre) # suppression du rectangle Format vectoriel
			except: pass
			return #et on ne fait rien

		x1,y1,x2,y2 = box
		bitmap = self.imageBitmap.copy()
		#creation de la nouvelle image
		draw = ImageDraw.Draw(bitmap, mode="RGB") #creation de l'instance ImageDraw qui va permettre de dessiner
		color = "rgb"+str(getrgb(self.graphColor))
		#on cherche  pallier le bug absurde du traage des lignes de PIL 1.1.6
		#FIXME : bug sur l'paisseur du trait mme en PIL 1.1.6
		#il est dommage que l'on ne puisse pas utiliser :
		#self.canvas.create_polygone(x1,y1,x2,y1,x2,y2,x1,y2,fill=color,width=widthGraph,outline=color)
		widthV = self.widthGraph 
		widthH = self.widthGraph
		if self.widthGraph == 1:       
			gap = 0
		elif (self.widthGraph % 2) == 1: #taille de l'paisseur impaire
			gap = self.widthGraph//2+1
			widthV = self.widthGraph-1
		else: #taille de l'paisseur paire
			gap = self.widthGraph//2
		if self.widthGraph == 11:
			widthV = self.widthGraph-2

		draw.line((x1,y1,x2,y1),fill=color,width=widthH)
		draw.line((x2,y1-gap,x2,y2+gap),fill=color,width=widthV)
		draw.line((x1,y1-gap,x1,y2+gap),fill=color,width=widthV)
		draw.line((x1,y2,x2,y2),fill=color,width=widthH)
		del draw #on a plus besoin de l'instance ImageDraw

		#pour contourner un bug non identifi, on excute la ligne suivante en try
		try: self.canvas.delete(self.cadre) #effacement du dernier cadre de mise en valeur vectoriel
		except: pass
		self.reconfigure(bitmap=bitmap)

	def getsizes(self, event=None):
		"""pour le debug"""
		print "main             h:   %i,\tL:   %i" % (self.winfo_height(),self.winfo_width())
		print "menu           h:   %i,\tL:   %i" % (self.menu.winfo_height(),self.menu.winfo_width())
		print "canvas           h:   %i,\tL:   %i" % (self.canvas.winfo_height(),self.canvas.winfo_width())
		print "canvas   cget    h:   %s,\tL:   %s" % (self.canvas.cget("height"),self.canvas.cget("width"))
		print "canvasHeight,Width:   %i,\t     %s" % (self.canvasHeight,self.canvasWidth)
		print "imagePhoto       h:   %i,\tL:   %i" % (self.imagePhoto.height(),self.imagePhoto.width())
		print "Scrollbar        h(x):%i,\tL(y):%i" % (self.xScrollbar.winfo_height(),self.yScrollbar.winfo_width())
		print "toolbar          h:   %i,\tL:   %i" % (self.toolbar.winfo_height(),self.toolbar.winfo_width())
		print "toolbar (bbox)    : ", self.toolbar.grid_bbox(column=0, row=2, col2=0, row2=2) 
		print

# MENU ET BARRE D'OUTILS #

	def createToolbar(self,parent):
		"""Interface de contrle de l'application bas sur des icones"""
		# interface application
		self.appliFrame = LabelFrame(parent, padx=2, pady=0,
				text="application")
		self.quitButton = ButtonBulle(self.appliFrame, delay=self.delay,
				command=self.exit, 
				icon = "icons/exit.png", 
				bulle="quitter")
		self.toolbarButton = ButtonBulle(self.appliFrame, delay=self.delay,
				command=self.hideToolbar, 
				icon = "icons/menu.png", 
				bulle="cacher la barre d'outils")

		# interface document
		self.editFrame = LabelFrame(parent, padx=2, pady=0,
				text="image")
		self.loadButton = ButtonBulle(self.editFrame, delay=self.delay,
				command=self.openClipboard,
				icon = "icons/load.png", 
				bulle="ouvrir le presse-papier")
		self.saveButton = ButtonBulle(self.editFrame, delay=self.delay,
				command=self.save, 
				icon = "icons/save.png", 
				bulle="sauver")
		self.undoButton = ButtonBulle(self.editFrame, delay=self.delay,
				command=self.undo, 
				icon="icons/undo.png", 
				bulle="annuler")
		self.sendButton = ButtonBulle(self.editFrame, delay=self.delay,
				command=self.send, 
				icon = "icons/wizard.png", 
				bulle="envoyer sur Mediawiki",
			 	background='red', foreground='white')

		# interface de mise en valeur
		self.graphFrame = LabelFrame(parent, padx=2, pady=0, 
				foreground=self.graphColor,
				text=self.widthGraphText)
		self.decWidthGraphButton = ButtonBulle(self.graphFrame, delay=self.delay,
				command=self.decWidthGraph, 
				icon="icons/minus.png", 
				bulle="encadrements plus fins")
		self.incWidthGraphButton = ButtonBulle(self.graphFrame, delay=self.delay,
				command=self.incWidthGraph, 
				icon="icons/plus.png", 
				bulle="encadrements plus pais") 
		self.colorGraphButton = ButtonBulle(self.graphFrame, delay=self.delay,
				command=self.changeColor, 
				icon="icons/color.png", 
				#background='red', foreground='black',
				bulle="couleur")

		#on place tous les widgets sur la grille
		self.appliFrame.grid(row=0,column=0,padx=15,pady=3)
		self.quitButton.grid(row=0,column=0,pady=2)
		self.toolbarButton.grid(row=0,column=1,pady=2)

		self.editFrame.grid(row=0,column=1,padx=15,pady=3)
		self.loadButton.grid(row=0,column=0,pady=2)
		self.saveButton.grid(row=0,column=1,pady=2)
		self.undoButton.grid(row=0,column=2,pady=2)
		self.sendButton.grid(row=0,column=3,pady=2)
		
		self.graphFrame.grid(row=0,column=2,padx=15,pady=3)
		self.decWidthGraphButton.grid(row=0,column=0,pady=2)
		self.incWidthGraphButton.grid(row=0,column=1,pady=2)
		self.colorGraphButton.grid(row=0,column=2,pady=2)

	def createMenu(self,parent):
		"""Interface de contrle de l'application bas sur un menu"""
		self.menu = Menu(parent)
		self.filemenu = Menu(self.menu, tearoff=1)
		self.filemenu.add_command(label="Ouvrir un fichier", command=self.openFile)
		self.filemenu.add_command(label="Sauvegarder", command=self.save)
		self.filemenu.add_separator()
		self.filemenu.add_command(label="Ouvrir le presse-papier", command=self.openClipboard)
		self.filemenu.add_command(label="Uploader", command=self.send)
		#TODO : fonctionnalit  coder si intressante: 
		#self.filemenu.add_command(label="Copier vers le presse-papier", command=self.saveClipboard)
		self.filemenu.add_separator()
		self.filemenu.add_command(label="Quitter", command=self.exit)
		self.menu.add_cascade(label="Session", menu=self.filemenu)

		self.editmenu = Menu(self.menu, tearoff=1)
		self.editmenu.add_command(label="Dfaire", command=self.undo)
		self.editmenu.add_separator()
		self.editmenu.add_command(label="Encadrements plus pais", command=self.incWidthGraph)
		self.editmenu.add_command(label="Encadrements plus fins", command=self.decWidthGraph)
		self.editmenu.add_command(label="Couleur des encadrements", command=self.selectColor)
		self.menu.add_cascade(label="Edition", menu=self.editmenu)

		self.configmenu = Menu(self.menu, tearoff=0)
		self.configmenu.add_command(label="Configurer Wiki'it", command=self.askWikiitConf)
		self.configmenu.add_command(label="Paramtres de l'image", command=self.askImageConf)
		self.configmenu.add_separator()
		self.configmenu.add_command(label="Afficher/cacher la barre d'outils", command=self.toolbarToggle)
		self.menu.add_cascade(label="Configuration", menu=self.configmenu)

		self.helpmenu = Menu(self.menu, tearoff=0)
		self.helpmenu.add_command(label="Manuel de Wiki'it", command=self.manual)
		self.helpmenu.add_command(label="A propos de Wiki'it", command=self.about)
		self.menu.add_cascade(label="Aide", menu=self.helpmenu)

	def backupImage(self):
		"""Backup l'image courante pour une restauration ultrieure"""
		if self.imageBitmap:
			self.old_imageBitmap = self.imageBitmap.copy()

# CALLBACKS #
	def scrollDown(self,event):  self.canvas.yview_scroll( 1, 'pages')
	def scrollUp(self,event):    self.canvas.yview_scroll(-1, 'pages')
	def scrollRight(self,event): self.canvas.xview_scroll( 1, 'pages')
	def scrollLeft(self,event):  self.canvas.xview_scroll(-1, 'pages')

	def scrollVert(self,event=None):
		"""gre les vnements de scrolling, pour windows"""
		if event:
			if event.delta > 0:
				self.scrollUp(event)
			elif event.delta < 0:
				self.scrollDown(event)

	def scrollHori(self,event=None):
		"""gre les vnements de scrolling, pour windows"""
		if event:
			if event.delta > 0:
				self.scrollLeft(event)
			elif event.delta < 0:
				self.scrollRight(event)

	def send(self,event=None):
		"""Exporte l'image cre, se connecte sur le serveur Wikimedia et lui envoie l'image"""
		if (self.conf.get('login') == "" or self.conf.get('passwd') == ""): #il manque un des 2 identifiants de connexion
			diag = authDialog(self, title="connexion Mediawiki",login=self.conf.get('login'),passwd=self.conf.get('passwd'))
			if diag.result == None:
				print "abandon"
				return None
			else: 
				self.conf.update(diag.result)
				if (self.conf.get('login') == "" or self.conf.get('passwd') == ""):
					showerror("Connexion","Donnes insuffisantes pour se connecter")
					return None 
			del diag
		#ici on va tenter de se connecter avec des identifiants (login, password)
		if self.mw and not self.mw.checkConn(): #on vrifie l'tat de la connexion via les cookies
			print "connexion invalide, ouverture nouvelle connexion"
			self.mw.close()
			self.mw = self.__openMW()
			if not self.mw:
				return None
		if not self.mw: #pas d'objet Mediawiki ouvert
			print "pas de connexion ouverte trouve, ouverture d'une connexion"
			self.mw = self.__openMW()
			if not self.mw:
				return None
		#ici on a donc une connexion ouverte avec des identifiants de connexion valides
		try: #on va crer un fichier temporaire mais on pourrait utiliser StringIO
			#file = NamedTemporaryFile(prefix="wikiit",suffix=".png")
			#self.imageBitmap.save(file,"PNG")
			#file.close()
			file, filename = mkstemp(prefix="wikiit",suffix=".png")
			os.close(file)
			self.imageBitmap.save(filename,"PNG")
		except OSError:
			showwarning("Impossible de crer un fichier temporaire dans %s ; vrifiez les droits !" %tempdir)
			return None
		#on saisit le nom de l'image distante et son descriptif
		if not self.confImage.getsafe("currFilename"):
			self.confImage.askConf(windowParent=self)
		destFilename = self.confImage.get("currFilename")	
		#on a une connexion ouverte, une image  envoyer, 
		#et un nom de fichier pour le futur fichier distant
		try: 
			#self.confImage._print()
			self.mw.upload(filename,
				destFilename,
				self.confImage.get("description"))
		except MWuploadExist, msg:
			showwarning("Upload",msg)
			self.confImage.incr() #on incrmente le compteur pour le mode pattern
			print "erreur d'upload sur %s (file exists)" % destFilename
			return None
		except MWuploadError, msg:
			showerror("Upload",msg)
			#on n'incrmente pas le compteur self.confImage.num
			print "erreur d'upload sur %s " % destFilename
			return None
		else:
			self.confImage.incr() #on incrmente le compteur pour le mode pattern
		os.remove(filename)

		#l'image a t uploade
		print "upload of %s" % destFilename
		self.sendButton.configure(background='green')
		self.sendButton.flash()
		self.sendButton.flash()
		print "L'image vient d'tre envoye sur l'hte distant"
		if sys.platform == "win32":  #on copie le lien wikimedia dans le clipboard
		#cf http://mrlauer.wordpress.com/2007/12/31/python-and-the-clipboard/
			if self.conf.get("back") == "link":
				clipSetText("[[Image:%s]]" % filedest)
			elif self.conf.get("back") == "name":
				clipSetText(filedest)
			else:
				print "modle de retour au clipboard inconnu"

	def __openMW(self):
		"""ouvre une connexion Mediawiki. Renvoie l'objet en cas de succs et rien sinon"""
		mw = mediawiki(self.conf.get('host')) 
		try: 
			mw.login(self.conf.get('login'),self.conf.get('passwd'))
		except MWauthError, msg:
			showwarning("Connexion",msg)
			#disqualification du mot de passe seul
			self.conf.set('passwd', "")
		except MWError, msg:
			showerror("Connexion",msg)
		else:
			return mw
	
	def exit(self,event=None):
		"""quitte l'application proprement"""
		self.quit() #quitte la boucle principale
		self.destroy() #detruit la fentre root et tous ses enfants

	def undo(self,event=None):
		"""Annule la dernire manipulation"""
		if (self.old_imageBitmap):
			#print "undo! "
			bitmap = self.old_imageBitmap.copy()
			self.reconfigure(bitmap=bitmap)
			#on discrdite l'image de backup car sinon c'est l'ancienne image courante
			self.old_imageBitmap = None 
	
	def save(self,event=None):
		"""Sauve le fichier sur disque"""
		#http://mail.python.org/pipermail/python-list/2007-July/448346.html
		filename = asksaveasfilename(parent=self,title="sauvegarder l'image",
			defaultextension = ".png", filetypes=[("Image PNG","*.png")])#,
			#initialdir = os.path.expanduser("~"))
		if filename: 
			try:
				self.imageBitmap.save(filename,"PNG")
			except IOError:
					showerror("Sauvegarde","Impossible de sauvegarder l'image")
			else:
				print "image sauvegarde dans %s" % filename
	
	def setWidthGraph(self,event=None,sign=None):
		"""Augmente ou diminue l'paisseur du trait du prochain encadrement"""
		if sign == "+":
			self.widthGraph = min(self.widthGraph+1,12)
		elif sign == "-":
			self.widthGraph = max(self.widthGraph-1,1)
		else:
			return
		self.widthGraphText = "paisseur: %i" % self.widthGraph
		self.graphFrame["text"] = self.widthGraphText

	def incWidthGraph(self,event=None):
		"""Augmente l'paisseur du trait du prochain encadrement"""
		self.widthGraph = min(self.widthGraph+1,12)
		self.widthGraphText = "paisseur: %i" % self.widthGraph
		self.graphFrame["text"] = self.widthGraphText

	def decWidthGraph(self,event=None):
		"""Diminue l'paisseur du trait du prochain encadrement"""
		self.widthGraph = max(self.widthGraph-1,1)
		self.widthGraphText = "paisseur: %i" % self.widthGraph
		self.graphFrame["text"] = self.widthGraphText

	def changeColor(self,event=None):
		"""Slectionne la couleur des prochains cadres"""
		try:
			self.graphColor = self.iterColor.next()
		except StopIteration:
			self.iterColor=iter(self.colors)
			self.graphColor = self.iterColor.next()
		#self.colorGraphButton["background"]=self.graphColor
		self.graphFrame["foreground"]=self.graphColor
	
	def selectColor(self,event=None):
		"""Charge une couleur depuis la palette RGB"""
		result = askcolor()
		if result != (None,None): 
			self.graphColor = str(result[1]) #result[1] est au format hexa #RRGGBB
			self.graphFrame["foreground"]=self.graphColor
	
	def openFile(self,event=None,filename=None):
		"""Charge une image depuis un fichier"""
		if filename == None: #on ouvre la boite de dialogue de slection de fichier
			filename = askopenfilename(parent=self,title="ouvrir l'image",
				defaultextension = ".png", filetypes=[("Image PNG","*.png")])#,
				#initialdir = os.path.expanduser("~"))
			if not filename:
				return
		try:
			bitmap = PILImage.open(filename).convert("RGB")
		except IOError:
			showerror("Chargement","fichier %s inexistant. Impossible de charger l'image" % filename)
		else:
			self.reconfigure(bitmap=bitmap)


	def showToolbar(self,event=None):
		"""montre la barre d'outils"""
		#self.toolbar.place(relx=0.5,rely=1,anchor=S)
		self.toolbar.grid(row=2, columnspan=2,sticky=S)#,fill=None,expand=False)
		self.conf.toolbar = "1"
		self.toolbarHeight = self.toolbar.winfo_height() # hauteur de la barre d'outils

	def hideToolbar(self,event=None):
		"""cache la barre d'outils"""
		#self.toolbar.place_forget()
		self.toolbar.grid_forget()
		self.conf.toolbar = "0"
		self.toolbarHeight = 0 # hauteur de la barre d'outils

	def toolbarToggle(self,event=None,show=None):
		"""Affiche/masque la barre d'outils"""
		if ((not show) and self.conf.toolbar == "0") or (show == "1"):
				self.showToolbar()
				self.canvasReconfig()
		elif ((not show) and self.conf.toolbar == "1") or (show == "0"):
				self.hideToolbar()
				self.canvasReconfig()

	def askWikiitConf(self):
		"""Appelle la boite de dialogue de configuration et ragit en fonction du rsultat"""
		self.conf.askConf(windowParent=self)
		self.toolbarToggle(show=self.conf.get('toolbar'))

	def askImageConf(self):
		"""Appelle la boite de dialogue de configuration des paramtres de l'image"""
		self.confImage.askConf(windowParent=self)
	
	def manual(self):
		"""Ouvre une fentre contenant un manuel concis"""
		texte="""Ceci est le manuel succint de Wiki'it

Wiki'it est une application prsentant une interface rapide entre 
le presse-papier et le serveur web Mediawiki dsign dans la configuration.
Elle procde en important des saisies d'cran, en permettant de les rogner,
d'en encadrer certaines zones, puis en uploadant l'image rsultante vers 
le serveur Mediawiki.
En cas d'upload russi, elle copie dans le presse-papier l'url Mediawiki 
rfrenant la nouvelle image, de sorte qu'on puisse la rfrencer dans 
un article Mediawiki simplement par copier/coller.

Raccourcis clavier:
Ctrl-v : charge le presse-papier
Ctrl-Enter : upload l'image
Ctrl-s : sauve l'image dans un fichier
Ctrl-q : quitte l'application
Ctrl-z : annule la dernire modification (1 niveau)
t : affiche/cache la barre d'outils
c : change la couleur des encadrements
+ : augmente l'paisseur des encadrements
- : diminue l'paisseur des encadrements
"""
		showinfo("manuel",texte)

	def about(self): 
		global version
		"""Ouvre une fentre contenant l' propos"""
		texte = """Wiki'it version """ + version + """
Copyright (c) 2008-2010, Jean-Philippe Ayanids <jp.ayanides@free.fr>
Ce programme est distribu selon les termes de la GPL v.3.0"""
		showinfo("A propos",texte)


##############################################################
class authDialog(Dialog):
	"""classe de type TopLevel, permettant d'afficher une double-zone de saisie pour l'identification"""
	def __init__(self, parent, title="Identification", login="", passwd=""):
		self.login, self.passwd = (login, passwd)
		self.fields = {}
		Dialog.__init__(self,parent,title=title)

	def body(self, master):
	#labels et placement des labels
		Label(master, text="login").grid(row=0)
		Label(master, text="password").grid(row=1)
	#champs
		self.fields['login']  = Entry(master)
		self.fields['passwd'] = Entry(master, show="*")
	#placement des champs
		self.fields['login'].grid(row=0, column=1)
		self.fields['passwd'].grid(row=1, column=1)
	#prinsertion des valeurs dans les champs
		self.fields['login'].insert(END,self.login)
		self.fields['passwd'].insert(END,self.passwd)
	#focus initial
		if self.login == "" :
			return self.fields['login']
		else:
			return self.fields['passwd']

	def apply(self):
		self.result = {}
		for k in self.fields.keys(): self.result[k] = self.fields[k].get()


##############################################################
if sys.platform == "win32": 
#manipulation du clipboard texte
	def clipGetText(): 
		win32clipboard.OpenClipboard() 
		text = win32clipboard.GetClipboardData(win32clipboard.CF_TEXT) 
		win32clipboard.CloseClipboard() 
		return text 
	  
	def clipSetText(string): 
		win32clipboard.OpenClipboard()
		win32clipboard.EmptyClipboard()
		win32clipboard.SetClipboardData(win32clipboard.CF_TEXT,string) 
		win32clipboard.CloseClipboard()


##############################################################
if __name__ == '__main__':
	logging.raiseExceptions = 0
	clip = WikiClip()
	clip.mainloop()
